==> RDP_0.2.0.tar.gz: RDP/src/wrapper.cpp <==
#include <Rcpp.h>
#include "RamerDouglasPeucker.h"

//' Ramer-Douglas-Peucker
//'
//' The [Ramer-Douglas-Peucker algorithm](https://en.wikipedia.org/wiki/Ramer-Douglas-Peucker_algorithm) for reducing the number of points on a curve.
//'
//' @details If there are no more than two points it does not make sense to simplify.
//' In this case the input is returned without further checks of `x` and `y`.
//'
//' @param x The `x` values of the curve as a vector.
//' @param y The `y` values of the curve as a vector.
//' @param epsilon The threshold for filtering outliers from the simplified curve.
//'
//' @return A `data.frame` with `x` and `y` values of the simplified curve.
//'
//' @examples
//' RDP::RamerDouglasPeucker(x = c(0, 1, 3, 5), y = c(2, 1, 0, 1), epsilon = 0.5)
//'
//' @export
//'
// [[Rcpp::export]]
Rcpp::DataFrame RamerDouglasPeucker(Rcpp::NumericVector x, Rcpp::NumericVector y, double epsilon)
{
    if (epsilon < 0 || Rcpp::NumericVector::is_na(epsilon))
        throw std::domain_error("epsilon must be a non-negative number");

    R_xlen_t nPoints = x.length();
    if (nPoints != y.length())
        throw std::invalid_argument("x and y vectors must be of equal length");

    if (nPoints <= 2)
        return Rcpp::DataFrame::create(Rcpp::Named("x") = x, Rcpp::Named("y") = y);

    std::vector<rdp::Point2D> points;
    points.reserve(nPoints);
    for (R_xlen_t i = 0; i != nPoints; ++i)
    {
        if (Rcpp::NumericVector::is_na(x[i]) || Rcpp::NumericVector::is_na(y[i]))
            throw std::invalid_argument("NA values are not allowed in coordinates");

        points.push_back({x[i], y[i]});
    }

    // We don't know the number of indices to keep a priori, but the first point is always kept
    std::vector<size_t> indicesToKeep;
    indicesToKeep.reserve(nPoints);
    indicesToKeep.push_back(0);

    rdp::RamerDouglasPeucker(points, epsilon * epsilon, 0, nPoints - 1, indicesToKeep);

    // Here initialization is faster than reserve + push_back
    size_t nIndices = indicesToKeep.size();
    std::vector<double> xOut(nIndices);
    std::vector<double> yOut(nIndices);

    for (size_t i = 0; i != nIndices; ++i)
    {
        size_t index = indicesToKeep[i];
        xOut[i] = x[index];
        yOut[i] = y[index];
    }

    return Rcpp::DataFrame::create(Rcpp::Named("x") = xOut, Rcpp::Named("y") = yOut);
}

==> RDP_0.2.3.tar.gz: RDP/src/wrapper.cpp <==
#include <Rcpp.h>
#include "RamerDouglasPeucker.h"

//' Ramer-Douglas-Peucker
//'
//' The [Ramer-Douglas-Peucker algorithm](https://en.wikipedia.org/wiki/Ramer-Douglas-Peucker_algorithm) for reducing the number of points on a curve.
//'
//' @details If there are no more than two points it does not make sense to simplify.
//' In this case the input is returned without further checks of `x` and `y`.
//'
//' @param x The `x` values of the curve as a vector.
//' @param y The `y` values of the curve as a vector.
//' @param epsilon The threshold for filtering outliers from the simplified curve.
//'
//' @return A `data.frame` with `x` and `y` values of the simplified curve.
//'
//' @examples
//' RDP::RamerDouglasPeucker(x = c(0, 1, 3, 5), y = c(2, 1, 0, 1), epsilon = 0.5)
//'
//' @export
//'
// [[Rcpp::export]]
Rcpp::DataFrame RamerDouglasPeucker(Rcpp::NumericVector x, Rcpp::NumericVector y, double epsilon)
{
    if (epsilon < 0 || Rcpp::NumericVector::is_na(epsilon))
        throw std::domain_error("epsilon must be a non-negative number");

    R_xlen_t nPoints = x.length();
    if (nPoints != y.length())
        throw std::invalid_argument("x and y vectors must be of equal length");

    if (nPoints <= 2)
        return Rcpp::DataFrame::create(Rcpp::Named("x") = x, Rcpp::Named("y") = y);

    std::vector<rdp::Point2D> points;
    points.reserve(nPoints);
    for (R_xlen_t i = 0; i != nPoints; ++i)
    {
        if (Rcpp::NumericVector::is_na(x[i]) || Rcpp::NumericVector::is_na(y[i]))
            throw std::invalid_argument("NA values are not allowed in coordinates");

        points.push_back({x[i], y[i]});
    }

    // We don't know the number of indices to keep a priori, but the first point is always kept
    std::vector<std::size_t> indicesToKeep;
    indicesToKeep.reserve(nPoints);
    indicesToKeep.push_back(0);

    rdp::RamerDouglasPeucker(points, 0, nPoints - 1, epsilon * epsilon, indicesToKeep);

    // Here initialization is faster than reserve + push_back
    std::size_t nIndices = indicesToKeep.size();
    std::vector<double> xOut(nIndices);
    std::vector<double> yOut(nIndices);

    for (std::size_t i = 0; i != nIndices; ++i)
    {
        std::size_t index = indicesToKeep[i];
        xOut[i] = x[index];
        yOut[i] = y[index];
    }

    return Rcpp::DataFrame::create(Rcpp::Named("x") = xOut, Rcpp::Named("y") = yOut);
}
